#include "HUDController.h"
#include "SDL_ttf.h"
#include "CSRPG.h"

/*
 * 	HUDController.cpp
 * 	Contains the constructor and functions for the 
 * 	HUDController class used for displays and text boxes
 * 	in CSRPG
 *
 * 	Changelog:
 * 		Mike, Apr 30


	Memory leaks fixed
 */

// Spell costs
#define HEAL_I_COST 5

// Functionality
void CGame::initFonts() {
	// initialize the font control
	printf("HUDController()\n");
	printf("Initializing font setup\n");
	if(TTF_Init() < 0) {
		fprintf(stderr,"SDL_ttf couldn't be initialized!!!\n");
		exit(1);
	}
	printf("Font init, loading fonts...\n");
	font = TTF_OpenFont("data/font.ttf",18);
	if(!font) {
		fprintf(stderr,"Could not load data/font.ttf\n");
		exit(1);
	}
	printf("Fonts loaded. OK\n");
}

void CGame::Fight_magicMenu() {
	/*
			bHasMagic = 0;
	bHasHealSpell = 0;
	bHasFireSpell = 0;
	bHasLightningSpell = 0;
	bHasEarthSpell = 0;
	bHasMeteor = 0;
	bHasHandOfDeath = 0;
	bHasFullHeal = 0;

	// special spells
	bHasScornAndAbuse = 0;
	bHasMessiah = 0;
	bHasTheRapture = 0;*/

	SDL_Rect *menuRect = new SDL_Rect();
	// centre the menu on screen
	menuRect->h = 340;
	menuRect->w = 320;
	menuRect->x = (RES_X - 320) / 2;
	menuRect->y = (RES_Y - 340) / 2;
	//SDL_FillRect(screen,menuRect,SDL_MapRGB(screen->format,0,0,100));
	// now draw the text in the menu, according to what spells you have in your inventory
	SDL_Rect *textRect = new SDL_Rect();
	textRect->x = menuRect->x + 10;
	textRect->y = menuRect->y + 10;
	SDL_Surface *temp;
	SDL_Color colour = {255,255,255,255};
	temp = TTF_RenderText_Solid(font,"Magic:",colour);
	SDL_BlitSurface(temp,NULL,screen,textRect);
	textRect->y += textRect->h;
	temp = TTF_RenderText_Solid(font,"1-0 to cast, ESC to abort",colour);
	SDL_BlitSurface(temp,NULL,screen,textRect);
	textRect->y += textRect->h;
	if(you.bHasHealSpell) {
		temp = TTF_RenderText_Solid(font,"1) Heal I",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	if(you.bHasFireSpell) {
		temp = TTF_RenderText_Solid(font,"2) Fire I",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	if(you.bHasLightningSpell) {
		temp = TTF_RenderText_Solid(font,"3) Mr. Zappy",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	if(you.bHasEarthSpell) {
		temp = TTF_RenderText_Solid(font,"4) Earth",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	if(you.bHasMeteor) {
		temp = TTF_RenderText_Solid(font,"5) Meteor",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	if(you.bHasHandOfDeath) {
		temp = TTF_RenderText_Solid(font,"6) Hand of Death",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	if(you.bHasFullHeal) {
		temp = TTF_RenderText_Solid(font,"7) Full Heal",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	if(you.bHasMessiah) {
		temp = TTF_RenderText_Solid(font,"8) Messiah",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	if(you.bHasTheRapture) {
		temp = TTF_RenderText_Solid(font,"9) The Rapture (WARNING: Everyone dies)",colour);
		SDL_BlitSurface(temp,NULL,screen,textRect);
		textRect->y += textRect->h;
	}
	SDL_Flip(screen);
	bool bChosen = 0;
	while(!bChosen) {
		while(SDL_PollEvent(&event)) {
			if(event.type == SDL_KEYDOWN) {
				if(event.key.keysym.sym == SDLK_ESCAPE) {
					bPlayerTurn = 1;
					bChosen = 1;
					return;
				}
				// interpret their hits, and then see if there's a command to use
				else if(event.key.keysym.sym == SDLK_1) {
					if(you.bHasHealSpell) {
						// use heal spell here.
						if(you.mp > HEAL_I_COST) {
							you.mp -= HEAL_I_COST;
							you.hp += rand() % 20;
							SDL_Rect rect;
							rect.x = 80;
							rect.y = RES_Y - 300;
							SDL_BlitSurface(fightEffects[1],NULL,screen,&rect);
							if(you.hp > you.hp_max) {
								you.hp = you.hp_max;
							}
							displayFightConsoleMessage("Healed!!!!",0);
							bChosen = 1;
							return;
						} else {
							displayFightConsoleMessage("Not enough energy to cast Heal I",1);
							return;
						}
					}
				}
				else if(event.key.keysym.sym == SDLK_2) {
					if(you.bHasFireSpell) {
						if(you.mp >= 8) {
							you.mp -= 8;
							// display graphics!
							SDL_Rect rect;
							rect.x = RES_X / 2 - 20;
							rect.y = RES_Y - 270;
							rect.w = 128;
							rect.h = 128;
							SDL_BlitSurface(fightEffects[2],NULL,screen,&rect);
							int dmg = rand() % 20;
							targetNPC->hp -= dmg;
							char *tempstr = new char[40];
							sprintf(tempstr,"Foom! Hit %i",dmg);
							displayFightConsoleMessage(tempstr,0);
							delete[] tempstr;
							bChosen = 1;
							return;
						} else {
							displayFightConsoleMessage("Not enough energy for Fire I",1);
							return;
						}
					}
				}
				else if(event.key.keysym.sym == SDLK_3) {
					if(you.bHasLightningSpell) {
						//SDL_Rect *
						if(you.mp >= 11) {
							you.mp -= 11;
							// display graphics.
							SDL_Rect rect;
							rect.x = (RES_X / 2) - 40;
							rect.y = RES_Y - 290;
							rect.w = 128;
							rect.h = 128;
							SDL_BlitSurface(fightEffects[3],NULL,screen,&rect);
							int dmg = rand() % 20 + 10;
							targetNPC->hp -= dmg;
							char *tempstr = new char[40];
							sprintf(tempstr,"Foom! Hit %i",dmg);
							displayFightConsoleMessage(tempstr,0);
							delete[] tempstr;
							bChosen = 1;
							return;
						} else {
							displayFightConsoleMessage("Not enough energy!",1);
						}
					}
				}
				else if(event.key.keysym.sym == SDLK_4) {
					if(you.bHasEarthSpell) {
						if(you.mp >= 15) {
							you.mp -= 15;
							// display graphics.
							SDL_Rect rect;
							rect.x = (RES_X / 2) - 40;
							rect.y = RES_Y - 290;
							rect.w = 128;
							rect.h = 128;
							SDL_BlitSurface(fightEffects[4],NULL,screen,&rect);
							int dmg = rand() % 20 + 15;
							targetNPC->hp -= dmg;
							char *tempstr = new char[40];
							sprintf(tempstr,"Thud! Hit %i",dmg);
							displayFightConsoleMessage(tempstr,0);
							delete[] tempstr;
							bChosen = 1;
							return;
						} else {
							displayFightConsoleMessage("Not enough energy!",1);
							return;
						}
					}
				}
				else if(event.key.keysym.sym == SDLK_5) {
					if(you.bHasMeteor) {
						if(you.mp >= 20) {
							you.mp -= 25;
							// display graphics.
							SDL_Rect rect;
							rect.x = (RES_X / 2) - 40;
							rect.y = RES_Y - 290;
							rect.w = 128;
							rect.h = 128;
							SDL_BlitSurface(fightEffects[5],NULL,screen,&rect);
							int dmg = rand() % 20 + 20;
							targetNPC->hp -= dmg;
							char *tempstr = new char[40];
							sprintf(tempstr,"KABOOM!!!!!! Hit %i",dmg);
							displayFightConsoleMessage(tempstr,0);
							delete[] tempstr;
							bChosen = 1;
							return;
						} else {
							displayFightConsoleMessage("Not enough energy!",1);
							return;
						}
					}
				}
				else if(event.key.keysym.sym == SDLK_6) {
					if(you.bHasHandOfDeath) {
						if(you.mp >= 25) {
							you.mp -= 25;
							// display graphics.
							SDL_Rect rect;
							rect.x = (RES_X / 2) - 40;
							rect.y = RES_Y - 320;
							rect.w = 128;
							rect.h = 128;
							SDL_BlitSurface(fightEffects[6],NULL,screen,&rect);
							int dmg = rand() % 20 + 30;
							targetNPC->hp -= dmg;
							char *tempstr = new char[40];
							sprintf(tempstr,"Whap! Hit %i",dmg);
							displayFightConsoleMessage(tempstr,0);
							delete[] tempstr;
							bChosen = 1;
							return;
						} else {
							displayFightConsoleMessage("Not enough energy!",1);
							return;
						}
					}
				}
				else if(event.key.keysym.sym == SDLK_7) {
					if(you.bHasFullHeal) {
						if(you.mp >= 35) {
							you.mp -= 35;
							you.hp = you.hp_max;
							SDL_Rect rect;
							rect.x = 100;
							rect.y = RES_Y - 300;
							rect.w = 128;
							rect.h = 128;
							SDL_BlitSurface(fightEffects[7],NULL,screen,&rect);
							displayFightConsoleMessage("Fully healed",0);
							bChosen = 1;
							return;
						} else {
							displayFightConsoleMessage("Not enough energy!",1);
							return;
						}
					}
				}
				else if(event.key.keysym.sym == SDLK_8) {
					if(you.bHasMessiah) {
						displayFightConsoleMessage("Messiah triggers automatically...",0);
					}
				}
				else if(event.key.keysym.sym == SDLK_9) {
					if(you.bHasTheRapture) {
						// It's over....
						theRapture();
						spawnMrQuesty();
						list <NPC*>::iterator nit;
						nit = characters.begin();
						NPC *mrQuesty = *nit;
						mrQuesty->bIsMrQuesty = 0;
						mrQuesty->x = you.x - 1;
						mrQuesty->y = you.y - 1;
						mrQuesty->whatToSay0 = "What have you done?";
						bChosen = 1;
						nit++;
						NPC *mrShoppy = *nit;
						characters.remove(mrShoppy);
						delete mrShoppy;
						return;
					}
				}
			}
		}
	}
	SDL_Delay(1000);

}

HUDController::HUDController() {
	qm = new QuestMaster(); // make sure that we're not overrunning something
}

HUDController::~HUDController() {
	TTF_Quit();
	delete qm;
}

void CGame::printYourMoney(int money) {
	printf("printYourMoney(): this function is deprecated.\n");
	printf("\t\t\tdon't be a moron, inhale some boron\n");
}

void CGame::theRapture() {
	displayFightConsoleMessage("Everyone has died.",1);
	//SDL_BlitSurface(fi
	SDL_Rect rect = {0,0,0,0};
	SDL_BlitSurface(fightEffects[10],NULL,screen,&rect);
	SDL_Flip(screen);
	SDL_Delay(3000);
	for(int i = 255; i > 0; i--) {
		//SDL_Color colour = {i,i,i,i};
		SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format,i,i,i));
		SDL_Flip(screen);
		SDL_Delay(20);
	}
	// ok, let's go to the epilogue...
	//SDL_Delay(2000);
	//displayFightConsoleMessage("Press ESCAPE to survive the ruined worl
	for(int r = 0; r < MAP_Y_SIZE; r++) {
		for(int c = 0; c < MAP_X_SIZE; c++) {
			int res = rand() % 30;
			if(res == 0) {
				theMap.level[r][c] = '0'; // grass patch
			} else if(res == 1) {
				theMap.level[r][c] = '1'; // wall
			} else {
				theMap.level[r][c] = '4'; // desolate wasteland
			}
		}
	}
	bIsFighting = 0;
	clearNPCs();
	return;
}


void CGame::displayFightConsoleMessage(char *str, bool bIsBad) {
	SDL_Rect box;
	box.x = 0;
	box.y = RES_Y - 100;
	box.w = RES_X;
	box.h = 30;
	if(bIsBad) {
		SDL_FillRect(screen,&box,SDL_MapRGB(screen->format,255,0,0));
	} else {
		// if you hit the enemy, hooray, the box will be blue
		SDL_FillRect(screen,&box,SDL_MapRGB(screen->format,0,0,200));
	}
	box.x += 10;
	SDL_Color colour;
	colour.r = 255;
	colour.g = 255;
	colour.b = 255;
	SDL_Surface *temp = TTF_RenderText_Solid(font,str,colour);
	box.y -= 2;
	SDL_BlitSurface(temp,NULL,screen,&box);
	SDL_Flip(screen); // update the screen (this should be called LAST)
	// wait for the user to strike a key and then dispense with this message
	bool bDone = 0;
	SDL_Event event;
	while(!bDone) {
		while(SDL_PollEvent(&event)) {
			// pause for input
			if(event.type == SDL_KEYDOWN) {
				if(event.key.keysym.sym == SDLK_F4) {
					printf("Screenshot\n");
					SDL_SaveBMP(screen,"screenshot.bmp");
				} else {
					bDone = 1;
				}
			}
		}
	}
}

void CGame::printItemInfo(string Name, int cost) {
	/*
		Causes memory leak also 
	*/
	SDL_Rect *box = new SDL_Rect();
	box->h = 32;
	box->x = 10;
	box->y = RES_Y - box->h;
	box->w = RES_X;
	SDL_Color colour;
	colour.r = 255;
	colour.g = 255;
	colour.b = 255;

	// FIXME: Here is where the pain starts
	char *foo = new char[50];
	sprintf(foo,"You have $%i",you.gold);
	SDL_Surface *temp;
	temp = TTF_RenderText_Solid(font,(const char*)foo,colour);
	if(!temp) {
		printf("Might be out of memory...\n");
	}
	SDL_BlitSurface(temp,NULL,screen,box);
	// And here's where it ends
	SDL_FreeSurface(temp);
	//delete temp;
	delete[] foo;
	delete box;
}

void CGame::drawItemsPotionsKeys() {
	// draw potions and keys in a little line at the bottom of the screen
	SDL_Rect *tempRect = new SDL_Rect();
	tempRect->x = 15;
	tempRect->y = RES_Y - 55;
	tempRect->w = 48;
	tempRect->h = 48;
	for(int i = 0; i < you.healingpotions; i++) {
		SDL_BlitSurface(items[0],NULL,screen,tempRect);
		tempRect->x += 49;
	}
	// mana
	tempRect->y -= 49;
	tempRect->x = 15;
#ifdef _WIN32
	for(i = 0; i < you.manapotions; i++) {
#else
	for(int i = 0; i < you.manapotions; i++) {
#endif
		SDL_BlitSurface(items[13],NULL,screen,tempRect);
		tempRect->x += 49;
	}
	// keys
	tempRect->y -= 49;
	tempRect->x = 15;
#ifdef _WIN32
	for( i = 0; i < you.keys; i++) {
#else
	for(int i = 0; i < you.keys; i++) {
#endif
		SDL_BlitSurface(items[2],NULL,screen,tempRect);
		tempRect->x += 49;
	}
	//tempRect->y -= 
}

Quest* CGame::initiateConversation(NPC *plyr, int map) {
	
	// create an SDL rect to store the conversation bit.
	SDL_Rect *box = new SDL_Rect();
	box->h = 20;
	box->w = RES_X;
	box->x = 0;
	box->y = RES_Y - box->h;
	// now, paint the box on screen, print the text and wait for input
	SDL_FillRect(screen,box,SDL_MapRGB(screen->format,0,0,200));
	SDL_Color colour;
	colour.r = 255;
	colour.g = 255;
	colour.b = 255; // white
	box->y -= 20;
	const char *name = plyr->myName.c_str();
	//sprintf(name,"%s\n",plyr->whatToSay0.c_str());
	SDL_Surface *temp = TTF_RenderText_Solid(font,(const char*)name,colour);
	box->y -= 2;
	SDL_BlitSurface(temp,NULL,screen,box); // draw the name
	box->y += 20;
	// SPRINTF does not like const char* anything
	const char *dialogue = "Dialog error";
	if(!plyr->bIsMrQuesty) {
		// Normal moron, let's see what they say
		dialogue = plyr->whatToSay0.c_str();
	} else {
		// Mr. Questy says obscene things into your ear.
		string temp = qm->GiveNextQuest(map);
		printf("Returned %s\n",temp.c_str()); // this returns alright
		plyr->whatToSay0 = temp;
		dialogue = plyr->whatToSay0.c_str();
	}
	temp = TTF_RenderText_Solid(font,(const char*)dialogue,colour);
	//delete dialogue;
	SDL_BlitSurface(temp,NULL,screen,box);
	bool bDone = 0;
	SDL_Flip(screen);
	SDL_Event event;
	while(!bDone) {
		while(SDL_PollEvent(&event)) {
			// pause for input
			if(event.type == SDL_KEYDOWN) {
				if(event.key.keysym.sym == SDLK_F4) {
					printf("Screenshot\n");
					SDL_SaveBMP(screen,"screenshot.bmp");
				} else {
					bDone = 1;
				}
			}
		}
	}
	if(plyr->bIsMrQuesty) {
		// still, free up some memory
		//delete dialogue;
		return qm->NextQuest(map);
	}
	SDL_FreeSurface(temp); // clean up after ourselves
	//delete temp;
	delete box; // clear up the SDL_Rect
	return NULL;
}

void CGame::displayQuestCompletion() {
	SDL_Rect rect = {0,0,0,0};
	rect.x = 20;
	rect.y = RES_Y / 2;
	SDL_Color colour = {255,255,255,255};
	SDL_Surface *temp = TTF_RenderText_Solid(font,"You completed a quest",colour);
	//SDL_Flip(screen);
	SDL_Event event;
	bool bDone = 0;
	SDL_BlitSurface(temp,NULL,screen,&rect);
	SDL_Flip(screen);
	while(!bDone) {
		while(SDL_PollEvent(&event)) {
			// pause for input
			if(event.type == SDL_KEYDOWN) {
				if(event.key.keysym.sym == SDLK_F4) {
					printf("Screenshot\n");
					SDL_SaveBMP(screen,"screenshot.bmp");
				} else {
					bDone = 1;
				}
			}
		}
	}
	SDL_FreeSurface(temp);
	//delete temp;
}

/*
void CGame::HUD_Player() {
	// THIS THING IS FULL OF THE MEMORY LEAK
	SDL_Rect *rect = new SDL_Rect();
	rect->x = 20;
	rect->y = 10;
	//const char HUD[100] = "empty"; // this probably uses GOBS of memory.
	char *HUD;
	if(!you.questptr) {
		sprintf((char*)HUD,"HP: %i/%i Gold: %i Keys: %i",you.hp,you.hp_max,you.gold,you.keys);
	} else {
		sprintf((char*)HUD,"HP: %i/%i Gold: %i Quest: %s",you.hp,you.hp_max,you.gold,you.questptr->TheDescript.c_str());
	}
	SDL_Color color = {0,0,0,0};
	SDL_Surface *temp = TTF_RenderText_Solid(font,(const char*)HUD,color);
	SDL_BlitSurface(temp,NULL,screen,rect);
	delete rect;
	//delete HUD;
	SDL_FreeSurface(screen);
	SDL_FreeSurface(temp);
	//delete screen; // delete our copy
}*/

void CGame::HUD_Player() {
	SDL_Rect rect;
	rect.w = you.hp_max * 2;
	rect.x = 20;
	rect.y = 10;
	rect.h = 21;
	SDL_Rect *iconRect = new SDL_Rect();
	iconRect->x = 0;
	iconRect->y = 0;
	SDL_FillRect(screen,&rect,SDL_MapRGB(screen->format,0,0,0));
	rect.w = you.hp * 2;
	SDL_FillRect(screen,&rect,SDL_MapRGB(screen->format,100,0,0));
	SDL_BlitSurface(icons[3],NULL,screen,iconRect);
	char *foo = new char[30];
	SDL_Color colour = {255,255,255,255};
	sprintf(foo,"%i",you.hp);
	SDL_Surface *temp = TTF_RenderText_Solid(font,(const char*)foo,colour);
	rect.x += 10;
	SDL_BlitSurface(temp,NULL,screen,&rect);
	rect.x -= 10;
	//rect->x += 
	// crap, here's the magic bar
	rect.x = 20 + you.hp_max * 2 + 5;
	rect.w = you.mp_max * 2;
	sprintf(foo,"%i",you.mp);
	temp = TTF_RenderText_Solid(font,(const char*)foo,colour);
	rect.h = 21;
	SDL_FillRect(screen,&rect,SDL_MapRGB(screen->format,0,0,0));
	rect.w = you.mp * 2;
	rect.h = 21;
	SDL_FillRect(screen,&rect,SDL_MapRGB(screen->format,0,100,0));
	SDL_BlitSurface(temp,NULL,screen,&rect);
}

void CGame::HUD_Enemy(NPC *plyr) {
	printf("Initializing rectangle\n");
	SDL_Rect rect = {0,0,0,0};
	rect.x = 20;
	rect.y = 30;
	printf("Initializing colour\n");
	SDL_Color colour = {255,255,255,255};
	colour.g = 0;
	colour.b = 100;
	printf("allocating dialogue memory\n");
	//const char dialogue[100] = "Dialog error";
	char *dialogue = new char[100];

	//const char *HUD = "Foo";
	printf("sprintf operation\n");
	sprintf(dialogue,"%s HP: %i/%i",plyr->myName.c_str(),plyr->hp,plyr->hp_max);
	SDL_Surface *temp = TTF_RenderText_Solid(font,(const char*)dialogue,colour);
	if(!temp) {
		fprintf(stderr,"Failure in rendering text.\n");
		exit(1);
	}
	rect.w = temp->w;
	rect.h = temp->h;
	printf("Mapped rect, blitting...\n");
	SDL_BlitSurface(temp,NULL,screen,&rect);
	//delete[] HUD;
	//delete HUD;
	SDL_FreeSurface(temp);
	delete[] dialogue;
	printf("Memory freed, HUD_Enemy exiting\n");
}

