#ifndef _PLAYER_H_
#define _PLAYER_H_

#include "npc.h"
#include "Quests.h"
#include "Shops.h"
#include <vector>
using namespace std;

class Quest;

/// Brief documentation for Player.
/** Lengthy documentation for our player. */
class Player : public NPC {
	public:
	       /// a pointer to their current objective
		Quest *questptr;
		/// the number of the "level" they're on, starting from 0
		int map;
	       /// how much money the player has
		int gold; // how many quests have you done?
		/// obviously how many skeleton keys our player is carrying
		int keys; 
		/// moves the player relatively, checks to see if it completes a quest.
		/** returns '1' if your movement did complete a quest. */
		int moveRelative(int dx, int dy); 

		/// How many "experience points" the player has
		/** Every multiple of ten is automatically checked to see if you get a new level */
		int experience;

		// Items
		/// Your current weapon modifier. Affects damage and accuracy
		int weapon_mod;
		/// Your current armour class. Affects dodge and damage dampening.
		int ac_mod; 
		/// Current strength base, added to weapon_mod. Unused for now.
		int strength_mod;
		/// Current agility base. Added to ac_mod
		int agility_mod;
		/// Number of healing potions in inventory.
		int healingpotions;
		/// Number of mana potions in inventory
		int manapotions;

		// Spell inventory
		/// Has magic capability, unused
		bool bHasMagic;
		/// Has Heal 1 spell, by default
		bool bHasHealSpell;
		/// Has Fire 1 spell
		bool bHasFireSpell;
		/// Has Mr. Zappy spell
		bool bHasLightningSpell;
		/// Has Earth 1 spell
		bool bHasEarthSpell;
		/// Has Meteor 1 spell
		bool bHasMeteor;
		/// Has Hand of Death spell
		bool bHasHandOfDeath;
		/// Has Full Heal spell
		bool bHasFullHeal;
		/// Has Scorn & Abuse spell, obtained at level 3
		/** Damaging attack */
		bool bHasScornAndAbuse;
		/// Has Messiah spell, obtained at level 6.
		/** Allows you to come back from the dead */
		bool bHasMessiah;
		/// Has the Rapture spell, obtained at level 9.
		/** Kills everyone in the game and leaves you alone to ponder a ruined world of your own creation. */
		bool bHasTheRapture;

		/// Our player's inventory, made of Shop items.
		std::vector <Item*> yourStuff;

		/// Default variables and statistics for our player.
		Player();
};
#endif
