#ifndef _QUESTS_H_
#define _QUESTS_H_

#include <vector>
#include "npc.h"
#include "Player.h"
#include <string>
using namespace std;

#define GOLD_PER_QUEST 40 // how many gold pieces mr. questy gives you for beating a quest

/// A generic class to use in the QuestMaster.
/** This class is basically a 'quest', includes conditions for
 * completion. The game stores a vector full of these to provide
 * the "backbone" for Mr. Questy.
 */
class Quest {
public:
	/// The x,y location that will complete the quest
	int x,y; 
	/// The floor the quest is on. Quests will only be given out on the floor they can be completed on, for simplicity.
	int map; 
	/// Is the quest completed?
	bool bDone;
	/// What is the one-line description of this quest? STL String
	string TheDescript;
	/// Will completing this quest give me an additional skeleton key?
	bool bGivesKey; 
	/// Constructor for default values.
	Quest();
};

/// A class that holds a lot of Quests, and allocates them as fit.
/** This class basically tracks the player's progress and gives them
 * undone quests. It also stores all of the quests, and can load them
 * from a local text file. See Quests.cpp for the load code.
 */
class QuestMaster {
public:
	/// Holds all the quests in the game right now.
	vector <Quest*> quests;
	/// Hands out the info for a quest for a certain floor, or returns BS if there is none
	string GiveNextQuest(int map);
	/// Loads quests from a file. Can be called multiple times to append quests vector.
	void loadQuests(char *filename);
	/// Gives a ptr to the next valid quest on this floor.
	Quest* NextQuest(int map);
	/// Constructor -- pretty much default values.
	QuestMaster();
};

#endif
