/*
	Shops.h
	Manages shop-type stuff.

*/

#ifndef _SHOPS_H_
#define _SHOPS_H_

#include <string>
#include <vector>
using namespace std;

/// A basic item sold in the shop.
/** Contains the cost, icon, modifiers (weapon and armour,
 * as well as special information (if it gives you a healing potion,
 * mana potion or spell) */
class Item {
public:
	// Generic item stuff
	/// how many gold points do you need to buy it?
	int cost; 
	/// what little picture should we fetch for it in store?
	int icon; 

	// Weapons-only stuff
	/// the weapons modifier.
	int mod; 
	/// the armour-class modifier.
	int ac_mod;
	/// the name of the item, in STL string
	string name; 

	/// does buying this item give you an additional healing potion
	bool bIsHealingItem;

	/// should we bother looking at the rest of the spells?
	bool bIsMagic;

	/// provides Heal 1
	bool bIsHealSpell;
	/// provides Fire 1
	bool bIsFireSpell;
	/// provides Mr. Zappy 1
	bool bIsLightningSpell;
	/// provides Earth 1
	bool bIsEarthSpell;
	/// provides Meteor 1
	bool bIsMeteor;
	/// provides Hand of Death 1
	bool bIsHandOfDeath;
	/// provides Full Heal 1
	bool bIsFullHeal;

	/// provides a skeleton key?
	bool bIsKey; 
	/// provides a magic-refreshing mana potion?
	bool bIsManaPotion; 

	/// how many of this item are left in store?
	/** We're hacking this up, always set to 1, but some enterprising
	 * hacker can probably add it to Shops.cpp's load item function */
	int count;

	/// Creates some default crap for items.
	Item();
};

/// A container to hold a lot of Items for sale.
/** The functionality for providing the buy/sell UI is actually
 * provided by the game, and an object of type Shop should be included
 * in the CGame class definition.
 *
 * Regardless, this holds a vector of items and their prices, and can
 * load them right from a local text file. See Shops.cpp for the load
 * code.
 */
class Shop {
public:
	/// holds all the items in the shop
	vector <Item*> shopItems; 
	/// loads items from a file.
	void loadItems(char *name); 
	/// builds basic crap, to prevent memory problems
	Shop();
};

#endif
