#include "CSRPG.h"
#include "SDL.h"
#include <time.h>
using namespace std;

/*
 * 	CSRPG.cpp
 * 	Implementation for CSRPG.h
 *
 * 	This file holds the constructor and destructor for the class
 * 	defined in CSRPG.h. Actual game functions are elsewhere, in the
 * 	interest of keeping this file relatively readable.
 *
 * 	Changelog:
 * 		Mike, Apr 30
		Mike, May 5 -- NPC Loader added, Corey will write an editor for end-users
 */

CGame::CGame() {
	menuchoice = -1; // checks and reset
	menuoff = -1;
	//MaxItems = 10;
	srand((unsigned int) time (NULL));
	qm = new QuestMaster();
		/*
		 * 	"I forgot"
		 * 	A haiku by Mike Stedman
		 *
		 * 	Oh, dynamic mem
		 * 	Why must you evade malloc probe?
		 * 	Oops I forgot new
		 */
	qm->loadQuests("data/quests.rav");
	/*
	 * Okay, set up the screen surface and check for hazards,
	 * 	then load in the graphics and levels and try to initialize
	 * 	everything required for the game state.
	 */
	atexit(SDL_Quit); // make sure the user isn't "trapped" in a resolution
			  // in full screen mode
	
	// set up SDL to begin rendering to our screen surface
	if(SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0) {
		fprintf(stderr,"CRITICAL: SDL didn't init %s\n",SDL_GetError());
		exit(1);
	}

	if(IS_FULLSCREEN == 0) {
		screen = SDL_SetVideoMode(RES_X,RES_Y,16,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_ANYFORMAT);
	} else {
		screen = SDL_SetVideoMode(RES_X,RES_Y,16,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_ANYFORMAT|SDL_FULLSCREEN);
	}

	if(!screen) {
		fprintf(stderr,"CRITICAL: Couldn't initialize screen surface: %s\n",SDL_GetError());
		exit(1);
	}

	SDL_WM_SetCaption("CS RPG","rav");

	// now load the sprites and tiles accordingly.
	printf("Loading sprites and tiles\n");
	loadSurfaces();
	printf("Sprites and tiles loaded.\n");

	// load the map file
	printf("Loading map...\n");
	theMap.loadMap("data/map.rav");
	printf("Map loaded.\n");

	// now set up miscellaneous variables
	bIsDone = 0;
	bIsFighting = 0;
	bPlayerTurn = 1;

	fightBackground = 0;

	// everything should have gone all right
	initFonts();

	targetNPC = NULL;

	/*	debug only */
	/*
	Item* MrFoobar = new Item();
	MrFoobar->mod = 0;
	MrFoobar->name = "Mr. Foobar";
	MrFoobar->ac_mod = 0;
	MrFoobar->icon = 0;
	you.yourStuff.push_back(MrFoobar);*/
	you.gold = 100000;
}

CGame::~CGame() {
	SDL_ShowCursor(1);
	//characters.clear()
	//SDL_FreeSurface(chargfx[
	for(int i = 0; i < MAX_SPRITES; i++) {
		for(int z = 0; z < 4; z++) {
			SDL_FreeSurface(chargfx[i][z]);
		}
	}
	you.yourStuff.clear();
	clearNPCs();
	TTF_Quit();
}

void CGame::loadSurfaces() {
	// ugly function alert!
	// avert your eyes NOW!!!!!!!!
	// unless you're adding a new png image to load.
	
	SDL_Surface	*temp;

	// load the sprites first
	
	chargfx[0][0] = IMG_Load("data/gfx/sprites/pc/0.png");
	chargfx[0][1] = IMG_Load("data/gfx/sprites/pc/1.png");
	chargfx[0][2] = IMG_Load("data/gfx/sprites/pc/2.png");
	chargfx[0][3] = IMG_Load("data/gfx/sprites/pc/3.png");

	if(chargfx[0][0] == NULL) {
		fprintf(stderr,"data/gfx/sprites/pc/0.png missing");
		exit(1);
	}

	chargfx[1][0] = IMG_Load("data/gfx/sprites/npc0/0.png");
	chargfx[1][1] = IMG_Load("data/gfx/sprites/npc0/1.png");
	chargfx[1][2] = IMG_Load("data/gfx/sprites/npc0/2.png");
	chargfx[1][3] = IMG_Load("data/gfx/sprites/npc0/3.png");

	if(chargfx[1][0] == NULL) {
		fprintf(stderr,"data/gfx/sprites/npc0/0.png missing\n");
		exit(1);
	}

	for(int i = 0; i < 4; i++) {
		// Mr. Questy
		chargfx[2][i] = IMG_Load("data/gfx/sprites/npc1/0.png");
	}

	
#ifdef _WIN32
	for(i = 0; i < 4; i++) {
#else
	for(int i = 0; i < 4; i++) {
#endif
		// Altar Boy
		chargfx[4][i] = IMG_Load("data/gfx/sprites/npc4/0.png");
	}

	// muntist priest
	chargfx[3][0] = IMG_Load("data/gfx/sprites/npc2/0.png");
	chargfx[3][1] = IMG_Load("data/gfx/sprites/npc2/1.png");
	chargfx[3][2] = IMG_Load("data/gfx/sprites/npc2/2.png");
	chargfx[3][3] = IMG_Load("data/gfx/sprites/npc2/3.png");

	// muntist "transcend"
	chargfx[5][0] = IMG_Load("data/gfx/sprites/npc5/0.png");
	chargfx[5][1] = IMG_Load("data/gfx/sprites/npc5/1.png");
	chargfx[5][2] = IMG_Load("data/gfx/sprites/npc5/2.png");
	chargfx[5][3] = IMG_Load("data/gfx/sprites/npc5/3.png");

	// just a regular muntist
	chargfx[6][0] = IMG_Load("data/gfx/sprites/npc3/0.png");
	chargfx[6][1] = IMG_Load("data/gfx/sprites/npc3/1.png");
	chargfx[6][2] = IMG_Load("data/gfx/sprites/npc3/2.png");
	chargfx[6][3] = IMG_Load("data/gfx/sprites/npc3/3.png");

	chargfx[7][0] = IMG_Load("data/gfx/sprites/npc6/0.png");
	chargfx[7][1] = IMG_Load("data/gfx/sprites/npc6/1.png");
	chargfx[7][2] = IMG_Load("data/gfx/sprites/npc6/2.png");
	chargfx[7][3] = IMG_Load("data/gfx/sprites/npc6/3.png");

	chargfx[8][0] = IMG_Load("data/gfx/sprites/npc7/0.png");
	chargfx[8][1] = IMG_Load("data/gfx/sprites/npc7/1.png");
	chargfx[8][2] = IMG_Load("data/gfx/sprites/npc7/2.png");
	chargfx[8][3] = IMG_Load("data/gfx/sprites/npc7/3.png");

	// mr. brown
	chargfx[9][0] = IMG_Load("data/gfx/sprites/npc8/0.png");
	chargfx[9][1] = IMG_Load("data/gfx/sprites/npc8/0.png");
	chargfx[9][2] = IMG_Load("data/gfx/sprites/npc8/0.png");
	chargfx[9][3] = IMG_Load("data/gfx/sprites/npc8/0.png");

	// if they're on a slow pc they'll never see this, so don't load it
	clouds = IMG_Load("data/gfx/clouds.jpg");
	if(!clouds) {
		fprintf(stderr,"data/gfx/clouds.jpg is missing\n");
		exit(1);
	}
	if(SLOW_PC == 1) {
		SDL_FreeSurface(clouds);
	}

#ifndef _WIN32
	// this is the new ANSI C++ standard
	for(int i = 0; i < MAX_SPRITES; i++) {
#else
	// improper according to the new ANSI C++ standard
	for(i = 0; i < MAX_SPRITES; i++) {
#endif
		for(int k = 0; k < 4; k++) {
			if(chargfx[i][k] == NULL) { 
				fprintf(stderr,"Character %i, heading %i missing\n",i,k);
				exit(1);
			}
			// do the alpha on this sprite,
			// and convert it to the display format of the screen
			temp = SDL_DisplayFormatAlpha(chargfx[i][k]);
			SDL_FreeSurface(chargfx[i][k]);
			chargfx[i][k] = temp;
		}
	}

	// now load the tiles. thankfully, no transparency req'd
	tiles[0] = IMG_Load("data/gfx/tiles/00.png");
	if(tiles[0] == NULL) {
		fprintf(stderr,"data/gfx/tiles/00.png missing!");
		exit(1);
	}

	tiles[1] = IMG_Load("data/gfx/tiles/01.png");
	if(tiles[1] == NULL) {
		fprintf(stderr,"data/gfx/tiles/01.png missing!");
		exit(1);
	}

	tiles[2] = IMG_Load("data/gfx/tiles/02.png");
	if(tiles[2] == NULL) {
		fprintf(stderr,"data/gfx/tiles/02.png missing!");
		exit(1);
	}

	tiles[3] = IMG_Load("data/gfx/tiles/03.png");
	if(tiles[3] == NULL) {
		fprintf(stderr,"data/gfx/tiles/03.png missing!\n");
		exit(1);
	}

	tiles[4] = IMG_Load("data/gfx/tiles/04.png");
	if(tiles[4] == NULL) {
		fprintf(stderr,"data/gfx/tiles/04.png missing!\n");
		exit(1);
	}

	tiles[5] = IMG_Load("data/gfx/tiles/05.png");
	if(tiles[5] == NULL) {
		fprintf(stderr,"data/gfx/tiles/05.png missing!\n");
		exit(1);
	}

	tiles[6] = IMG_Load("data/gfx/tiles/06.png");
	if(tiles[6] == NULL) {
		fprintf(stderr,"data/gfx/tiles/06.png missing!\n");
		exit(1);
	}

	tiles[7] = IMG_Load("data/gfx/tiles/07.png");
	if(tiles[7] == NULL) {
		fprintf(stderr,"data/gfx/tiles/07.png missing!\n");
		exit(1);
	}

	//currentfloor = 0;

#ifdef _WIN32
	for(i = 0; i < MAX_TILES; i++) {
#else
	for(int i = 0; i < MAX_TILES; i++) {
#endif
		// convert the tiles to the display "format" for fast
		// drawing.
		tiles[i] = SDL_ConvertSurface(tiles[i],screen->format,0);
	}

	icons[0] = IMG_Load("data/icn/fight.png");
	icons[1] = IMG_Load("data/icn/defend.png");
	icons[2] = IMG_Load("data/icn/magick.png");
	icons[3] = IMG_Load("data/icn/heart.png");
	icons[4] = IMG_Load("data/icn/manap.png");
	icons[5] = IMG_Load("data/gfx/scrollup.png");
	icons[6] = IMG_Load("data/gfx/scrolldown.png");
	icons[7] = IMG_Load("data/gfx/shop.png");
	icons[8] = IMG_Load("data/gfx/soldout.png");

#ifdef _WIN32
	// MICROSOFT WHY CAN'T YOU FOLLOW THE ISO C++ STANDARD
	// LIKE EVERY OTHER COMPILER CONSTRUCTOR ON THE PLANET?
	for(i = 0; i < 9; i++) {
#else
	for(int i = 0; i < 9; i++) {
#endif
		if(icons[i]) {
			temp = SDL_DisplayFormatAlpha(icons[i]);
			SDL_FreeSurface(icons[i]);
			icons[i] = temp;
		}
	}

	if(SLOW_PC != 1) {
		// if they're a slow pc, save 'em some memory,
		// because we ain't loading this anyway
		fightBackgrounds[0] = IMG_Load("data/decor/grass.png");
		fightBackgrounds[1] = IMG_Load("data/decor/inside.png");

	#ifdef _WIN32
		for(i = 0; i < 2; i++) {
	#else
		for(int i = 0; i < 2; i++) {
	#endif
			if(fightBackgrounds[i]) {
				fightBackgrounds[i] = SDL_ConvertSurface(fightBackgrounds[i],screen->format,0);
			} else {
				fprintf(stderr,"Fight scene background %i not located\n",i);
				exit(1);
			}
		}
	}

	// Something is definitely wrong here.
	//Quests crash, if this exists
	printf("Loading shop items icons\n");
	items.push_back(IMG_Load("data/gfx/items/0.png"));
	items.push_back(IMG_Load("data/gfx/items/1.png"));
	items.push_back(IMG_Load("data/gfx/items/2.png"));
	items.push_back(IMG_Load("data/gfx/items/3.png"));
	items.push_back(IMG_Load("data/gfx/items/4.png"));
	items.push_back(IMG_Load("data/gfx/items/5.png"));
	items.push_back(IMG_Load("data/gfx/items/6.png"));
	items.push_back(IMG_Load("data/gfx/items/7.png"));
	items.push_back(IMG_Load("data/gfx/items/8.png"));
	items.push_back(IMG_Load("data/gfx/items/9.png"));
	items.push_back(IMG_Load("data/gfx/items/10.png"));
	items.push_back(IMG_Load("data/gfx/items/11.png"));
	items.push_back(IMG_Load("data/gfx/items/12.png"));
	items.push_back(IMG_Load("data/gfx/items/13.png"));
	printf("Loaded shop icons\n");

#ifdef _WIN32
	for(i = 0; i < (int)items.size(); i++) {
#else
	for(int i = 0; i < items.size(); i++) {
#endif
		if(items[i]) {
			printf("Item %i is existant\n",i);
			temp = SDL_DisplayFormatAlpha(items[i]);
			SDL_FreeSurface(items[i]);
			items[i] = temp;
		} else {
			printf("Item load error, %i\n",i);
			SDL_Delay(200);
		}
	}

	portrait = IMG_Load("data/gfx/portrait.png");
	//delete temp;
	printf("Surfaces loaded...\n");

	// HARD CODED NPCs!!!! SOMEONE FIX THIS - MIKE

	// awright, now that we're done with the hardcoded characters,
	// let's load this crap in from file.
	printf("Loading NPCs!\n");
	loadNPCs("data/npcs.rav");
	printf("NPC load complete\n");

	//spawnMrQuesty();

// keep loading GFX
	fightEffects[0] = IMG_Load("data/gfx/fx/slash.png");
	fightEffects[1] = IMG_Load("data/gfx/fx/heal1.png");
	fightEffects[2] = IMG_Load("data/gfx/fx/fire1.png");
	fightEffects[3] = IMG_Load("data/gfx/fx/zappy.png");
	fightEffects[4] = IMG_Load("data/gfx/fx/earth.png");
	fightEffects[5] = IMG_Load("data/gfx/fx/meteor.png");
	fightEffects[6] = IMG_Load("data/gfx/fx/hand.png");
	fightEffects[7] = IMG_Load("data/gfx/fx/heal2.png");
	fightEffects[8] = IMG_Load("data/gfx/fx/scorn.png");
	fightEffects[9] = IMG_Load("data/gfx/fx/messiah.png");
	fightEffects[10] = IMG_Load("data/gfx/fx/rapture.png");
	for(int j = 0; j < MAX_EFFECTS; j++) {
		if(!fightEffects[j]) {
			fprintf(stderr,"missing %i fight fx\n",j);
			exit(1);
		}
		//fightEffects[j] =
		temp = SDL_DisplayFormatAlpha(fightEffects[j]);
		SDL_FreeSurface(fightEffects[j]);
		fightEffects[j] = temp;
	}

	// well, i think we're done here.

	SDL_ShowCursor(0);

	//SDL_FreeSurface(temp);
}

void CGame::clearNPCs() {
	list <NPC*>::iterator nit;
	nit = characters.begin();
	/*
		The old clearNPCs() call used to not clear out the entire list properly,
		but now it does. Mmmmmmmmm
	*/
	while(nit != characters.end()) {
		NPC* tn = *nit;
		nit++;
		characters.remove(tn);
		delete tn;
	}
	//characters.clear();
}

void CGame::spawnMrQuesty() {
	NPC* MrQuesty = new NPC();
	if(!MrQuesty) {
		fprintf(stderr,"Couldn't spawn Mr. Questy, maybe out of mem\n");
		exit(1);
	}
	MrQuesty->sprite = 2; // "Mr. Questy"
	MrQuesty->x = 2;
	MrQuesty->y = 2;
	MrQuesty->myName = "Mr. Questy";
	MrQuesty->bIsMrQuesty = 1;
	MrQuesty->whatToSay0 = "MR. QUESTY ERRORED";
	MrQuesty->heading = 2;
	characters.push_back(MrQuesty);

	NPC* ShopKeep = new NPC();
	if(!ShopKeep) {
		fprintf(stderr,"Couldn't spawn Shop Keeper, maybe out of mem\n");
		exit(1);
	}
	ShopKeep->sprite = 8; // Mr. Quest for now
	ShopKeep->x = 5;
	ShopKeep->y = 2;
	ShopKeep->myName = "Mr. Shoppy";
	ShopKeep->bIsShopKeeper = 1;
	ShopKeep->whatToSay0 = "Welcome to my shop!";
	ShopKeep->heading = 2;
	characters.push_back(ShopKeep);

}

void CGame::loadNPCs(char *filename) {
	printf("Opening file\n");
	ifstream TheFile(filename);
	if(!TheFile) {
		fprintf(stderr,"NPC file %s missing\n",filename);
		exit(1);
	}
	spawnMrQuesty(); // make sure he's there
	printf("Opened file\n");
	do {
		//printf("Constructing...\n");
		NPC* newNPC = new NPC();
		if(!newNPC) {
			fprintf(stderr,"Couldn't create new NPC: maybe out of mem\n");
			exit(1);
		}
		// okay, read in the next four values, in order they should be HP, HPMax, MP, MPMax
		char line[20] = "100";
		//printf("Beginning read\n");
		TheFile.getline(line,4,'\n');
		newNPC->hp = atoi(line);
		//printf("Loaded hp\n");
		TheFile.getline(line,4);
		newNPC->hp_max = atoi(line);
		//printf("Loaded hpmax\n");
		TheFile.getline(line,4);
		newNPC->mp = atoi(line);
		//printf("Loaded magick points\n");
		TheFile.getline(line,4);
		newNPC->mp_max = atoi(line);
		//printf("Loaded max magick\n");
		TheFile.getline(line,4);
		if(atoi(line) > 0) {
			//printf("Is hostile\n");
			newNPC->bIsHostile = 1;
		} else {
			newNPC->bIsHostile = 0;
		}
		char names[61] = "foo";
		TheFile.getline(names,60,'\n');
		char *tempc = names;
		newNPC->myName = tempc;
		//printf("Loaded name\n");
		TheFile.getline(names,60,'\n');
		newNPC->whatToSay0 = names;
		//printf("Loaded dialogue\n");
		TheFile.getline(line,4);
		newNPC->x = atoi(line);
		//printf("Loaded x-loc (%i)\n",newNPC->x);
		TheFile.getline(line,4);
		newNPC->y = atoi(line);
		//printf("Loaded y-loc (%i)\n",newNPC->y);
		TheFile.getline(line,4);
		newNPC->sprite = atoi(line);
		//printf("Loaded sprite\n");
		//printf("Loaded %s\n",newNPC->myName.c_str());
		newNPC->heading = 2; // load this in LATER
		if(newNPC->y == 0 && newNPC->x == 0) {
			printf("Panic mode: Null NPC probably received\n");
			TheFile.close();
			delete newNPC; // give us the memory back!!!!
			return;
		}
		if(TheFile.eof()) {
			// REALLY UGLY HACK TO FIX MY CRAP
			// Make sure that no NPC is ever specified at (0,0).
			printf("EOF detected. Returning..\n");
			newNPC->heading = 2;
			characters.push_back(newNPC);
			TheFile.close();
			return;
		}
		if(newNPC->y == 0 && newNPC->x == 0) {
			printf("Probably a null NPC, removing\n");
			TheFile.close();
			delete newNPC; // give us the memory back!!!
			return;
		}
		// cool, all loaded
		characters.push_back(newNPC);
	
		//TheFile.getline(line,4); // advances past EOF, or skips a line
		//getch();
	} while(!TheFile.eof());
	TheFile.close();
}

void CGame::solvedQuestScreen() {
	printf("Solved a quest\n");
	displayQuestCompletion();
	printf("Displayed quest completion screen\n");
}
