#ifndef _CSRPG_H_
#define _CSRPG_H_

/**
 * 	CSRPG.h
 * 	Main header file for the 2004 Computer Science AP project
 * 	by Mike Stedman, Corey Baker, and Geoff Jackson.
 *
 * 	This header file implements the basic "overlord" classes that
 * 	will make up the game engine. Look in CSRPG.cpp for the particular
 * 	implementation of such.
 */

// Graphic Settings -- Perhaps an enterprising young soul
// 			can make me a config file loader...
#define RES_X 640
#define RES_Y 480 // for "fight" backgrounds
#define IS_FULLSCREEN 1
#define CUTOFF_DISTANCE 8 // how far away from the player you draw tiles
#define SLOW_PC 0 // turns off backgrounds and such for 386's
#define INPUT_DELAY 60 // for performance testing, or if you have a 386, then set this to 0

#define CHEATER_MODE 1 // use the 'g' and 'k' keys to trade gold for keys and keys for gold
					  // (respectively)
#define MAX_EFFECTS 11

/*
	Current best performance with test first level, mp3 decoding in background, fullscreen,
	June 1st 2004:
		- DEBUG MODE: 40.26 FPS
		- RELEASE MODE: 42 FPS (Damn right!)

		Looks like the memory leak I cleared up did more than just fix our nasty nasty memory
		leaks :)
*/

// Defines for sprite directions -- this is the number the
// 				    game uses internally for each direction
// 				    sprites can have
#define FACING_UP 0
#define FACING_LEFT 1
#define FACING_DOWN 2
#define FACING_RIGHT 3

// Includes -- SDL (libsdl.org)
#include "SDL.h"
#include "SDL_image.h"
#include "SDL_mixer.h"
#include "SDL_ttf.h"

// Includes -- STL and ANSI C
#include <math.h>
#include <cstdlib>
#include <list>
#include <cstdio>

// Includes -- Our .h files
#include "npc.h"		// implements the NPC class
#include "HUDController.h"	// implements HUDController class
#include "Player.h"		// implements Player class
#include "Map.h"		// implements Map class
#include "SpecialTiles.h"	// defines for special tiles
#include "Shops.h"

// More defines -- graphics-based
#define MAX_TILES 8
#define MAX_SPRITES 10
#define MAX_ITEMS 9 // WHY DO YOU NOT WORK???#$?@

// Gameplay
#define MAX_FLOORS 5

// Misc.
#define IS_TRIPPY 0 // randomly shifts some colours

using namespace std;

/// Holds EVERYTHING for the game to run in a massive encapsulation.
/** Contains pretty much everything. The only functions that are
 * public are gameLoop() which should be called ASAP if you wish
 * to begin a game, after generating an object of type CGame.
 *
 * Oops, almost forgot: there's titleScreen() too so you can at least
 * display our title screen and loop if you want. I recommend not, it's
 * a very boring title.
 * */
class CGame {
	public:
		/// displays title screen
		void titleScreen(); 
		/// the game's main loop
		void gameLoop(); 
		/// the game's constructor; files/scripts loaded here
		CGame();
		/// the game's destructor; crap is cleared
		~CGame();
	private:
		// variables
		SDL_Surface *screen; // this holds the window we'll render to
		SDL_Event event; // this is used for input management
		Map theMap;	// this holds our stuff. look in Map.h for more
		//int currentfloor;
	
		// Generic variables
		bool bIsDone; // is the game over?
	
		// Game mode variables and functions
		bool bIsFighting; // are we in the fight engine?
		bool bPlayerTurn; // is the player ready to pick a menu option?
		/// the loop called when the player is walking around
		void RoamLoop();  
		/// the loop called when the player is fighting an NPC
		void FightLoop(); 
		/// this function is called to prepare the game engine for a new fight
		void prepareFight(NPC *enemy); 
		/// displays a small "you completed a quest" msg on screen
		void solvedQuestScreen();
		/// the debug console, activated from TAB, works in stdout/stdin
		void ConsoleLoop(); 	
		/* Generic functions */
		/// called from roamloop, draws tiles & sprites
		void Roam_DrawEverything(); 
			/// draws tiles to the screen
			void paintTiles(); 
			/// draws sprites, called each tile
			void drawSprite(int c, int r, SDL_Rect *temprect);
			/// checks around the player if an NPC is present to talk to
			void Roam_CheckNPCConvo(int x, int y);
		/// gets input from the player so they can walk around
		void Roam_GetInput();		
		//void Fight_GetInput();		// get input on menus
		/// loads NPCs from a file
		void loadNPCs(char *filename); 
		/// creates Mr. Questy and Mr. Shoppy at their default locs
		void spawnMrQuesty(); 
		/// deletes all of the NPCs and returns their memory
		void clearNPCs();
		/// checks a tile to see if it is a door.
		/** if it is a door, and you have a key, it's opened and a key is taken away. if you don't have any keys, it does nothing */
		void detectAndOpenDoors(int x, int y); 
		/// the shop menu
		void shopKeeping();

		// HUD repair functions
		/// draws the player's health and mana bars
		void HUD_Player();
		/// draws some info about the target NPC
		void HUD_Enemy(NPC* plyr);
		/// something or other that calls the solvedQuestScreen
		void displayQuestCompletion();
		/// draws the icons for the items you have to screen
		void drawItemsPotionsKeys();
		/// pops up a menu that you can pick what spell to use from
		void Fight_magicMenu();
		/// displays the player's stats and inventory. can equip from here
		void characterMenu();

		// Conversation repair functions
		/// Starts a conversation with a character.
		Quest* initiateConversation(NPC *plyr,int map);
		/// displays a message in a little info box that can be dismissed
		void displayFightConsoleMessage(char *str, bool bIsBad);
		/// prints information about an item highlighted in shop
		void printItemInfo(string name, int cost);
		/// deprecated; used to print how much $$$ you have
		void printYourMoney(int money); // for shops
		void theRapture(); // it's over

	
		// Scene graph, NPC lists.
		list <NPC*> characters;		// holds all NPC's
		Player you;			// hooray, you're useful!
		QuestMaster *qm; // holds all of our quests
		NPC* targetNPC; // the NPC you are currently fighting with!!!!
		int fightBackground; // the tile you are on will be saved and used in a fight
		Shop NormalShop;
		TTF_Font *font;
		int menuchoice;
		int menuoff; // menu offset (0 = render top x items)

		//int MaxItems;
		
		// Textures
		SDL_Surface *tiles[MAX_TILES];
		SDL_Surface *chargfx[MAX_SPRITES][4]; // the subscript is what direction they are facing
		SDL_Surface *icons[9];
		SDL_Surface *fightBackgrounds[2]; // for inside or outside
		SDL_Surface *clouds; // background
		SDL_Surface *portrait; // our player's portrait, 128x something
		SDL_Surface *fightEffects[MAX_EFFECTS];
		vector <SDL_Surface*> items;
	
		// Utility/Miscellaneous functions
		/// checks if an impassible tile is at (x,y)
		int tileCollision(int x, int y); 
			// returns 1 if there is an impassable tile at that location
		/// checks if a sprite is at (x,y)
		int spriteCollision(int x, int y); 
			// returns 1 if there is a character on that tile


		/// loads all surfaces into memory ready to go
		/** THIS FUNCTION IS REALLY UGLY */
		void loadSurfaces();
		/// loads all fonts into memory, ready to go
		void initFonts();
};

#endif
